@@ -23,15 +23,13 @@
  */
 
 /// @file
-///
-/// Forward declarations for all the C++ types used by Proton to
-/// represent AMQP types.
+/// Forward declarations for Proton types used to represent AMQP types.
 
 #include "./internal/config.hpp"
 
-#include <proton/type_compat.h>
-
-#include <string>
+#if PN_CPP_HAS_NULLPTR
+#include <cstddef>
+#endif
 
 namespace proton {
 
@@ -44,7 +42,6 @@ class symbol;
 class timestamp;
 class duration;
 class uuid;
-class uuid;
 class value;
 
 struct null {
