@@ -34,10 +34,9 @@ extern "C" {
 /**
  * @file
  *
- * Transport API for the proton Engine.
+ * @copybrief transport
  *
- * @defgroup transport Transport
- * @ingroup engine
+ * @addtogroup transport
  * @{
  */
 
@@ -68,42 +67,45 @@ typedef void (*pn_tracer_t)(pn_transport_t *transport, const char *message);
 #define PN_TRACE_OFF (0)
 
 /**
- * Log raw binary data into/out of the transport.
+ * Log raw binary data going in and out of the transport.
  */
 #define PN_TRACE_RAW (1)
 
 /**
- * Log frames into/out of the transport.
+ * Log protocol frames going in and out of the transport.
  */
 #define PN_TRACE_FRM (2)
 
 /**
- * Log driver related events, e.g. initialization, end of stream, etc.
+ * Log driver-related events.  For example, initialization, end of
+ * stream, and so on.
  */
 #define PN_TRACE_DRV (4)
 
 /**
- * Log events
+ * Log events.
  */
 #define PN_TRACE_EVT (8)
 
 /**
  * Factory for creating a transport.
+ *
  * A transport is used by a connection to interface with the network.
  * There can only be one connection associated with a transport. See
  * pn_transport_bind().
  *
- * Initially a transport is configured to be a client transport. Use pn_transport_set_server()
- * to configure the transport as a server transport.
+ * Initially a transport is configured to be a client transport. Use
+ * pn_transport_set_server() to configure the transport as a server
+ * transport.
  *
  * A client transport initiates outgoing connections.
  *
- * A client transport must be configured with the protocol layers to use and cannot
- * configure itself automatically.
+ * A client transport must be configured with the protocol layers to
+ * use and cannot configure itself automatically.
  *
- * A server transport accepts incoming connections. It can automatically
- * configure itself to include the various protocol layers depending on
- * the incoming protocol headers.
+ * A server transport accepts incoming connections. It can
+ * automatically configure itself to include the various protocol
+ * layers depending on the incoming protocol headers.
  *
  * @return pointer to new transport
  */
@@ -126,29 +128,34 @@ PN_EXTERN void pn_transport_set_server(pn_transport_t *transport);
  */
 PN_EXTERN void pn_transport_free(pn_transport_t *transport);
 
-/** Retrieve the authenticated user
+/**
+ * Retrieve the authenticated user.
  *
- * This is usually used at the the server end to find the name of the authenticated user.
- * On the client it will merely return whatever user was passed in to the
- * pn_connection_set_user() API of the bound connection.
+ * This is usually used at the the server end to find the name of the
+ * authenticated user.  On the client it will merely return whatever
+ * user was passed in to the pn_connection_set_user() API of the bound
+ * connection.
  *
- * The returned value is only reliable after the PN_TRANSPORT_AUTHENTICATED event has been received.
+ * The returned value is only reliable after the
+ * PN_TRANSPORT_AUTHENTICATED event has been received.
  *
  * @param[in] transport the transport
  *
- * @return
- * If a the user is anonymous (either no SASL layer is negotiated or the SASL ANONYMOUS mechanism is used)
- * then the user will be "anonymous"
- * Otherwise a string containing the user is returned.
+ * @return If a the user is anonymous (either no SASL layer is
+ * negotiated or the SASL ANONYMOUS mechanism is used) then the user
+ * will be "anonymous" Otherwise a string containing the user is
+ * returned.
  */
 PN_EXTERN const char *pn_transport_get_user(pn_transport_t *transport);
 
 /**
- * Set whether a non authenticated transport connection is allowed
+ * Set whether a non-authenticated transport connection is allowed.
+ *
+ * There are several ways within the AMQP protocol suite to get
+ * unauthenticated connections:
  *
- * There are several ways within the AMQP protocol suite to get unauthenticated connections:
  * - Use no SASL layer (with either no TLS or TLS without client certificates)
- * - Use an SASL layer but the ANONYMOUS mechanism
+ * - Use a SASL layer but the ANONYMOUS mechanism
  *
  * The default if this option is not set is to allow unauthenticated connections.
  *
@@ -172,8 +179,8 @@ PN_EXTERN bool pn_transport_is_authenticated(pn_transport_t *transport);
  * Set whether a non encrypted transport connection is allowed
  *
  * There are several ways within the AMQP protocol suite to get encrypted connections:
- * - Use TLS/SSL
- * - Use an SASL with a mechanism that supports saecurity layers
+ * - Use TLS
+ * - Use a SASL with a mechanism that supports security layers
  *
  * The default if this option is not set is to allow unencrypted connections.
  *
@@ -197,7 +204,7 @@ PN_EXTERN bool pn_transport_is_encrypted(pn_transport_t *transport);
  * Get additional information about the condition of the transport.
  *
  * When a PN_TRANSPORT_ERROR event occurs, this operation can be used
- * to access the details of the error condtion.
+ * to access the details of the error condition.
  *
  * The pointer returned by this operation is valid until the
  * transport object is freed.
@@ -208,9 +215,9 @@ PN_EXTERN bool pn_transport_is_encrypted(pn_transport_t *transport);
 PN_EXTERN pn_condition_t *pn_transport_condition(pn_transport_t *transport);
 
 /**
- * @deprecated
+ * **Deprecated**
  */
-PN_EXTERN  pn_error_t *pn_transport_error(pn_transport_t *transport);
+PN_EXTERN pn_error_t *pn_transport_error(pn_transport_t *transport);
 
 /**
  * Binds the transport to an AMQP connection.
@@ -234,6 +241,8 @@ PN_EXTERN int pn_transport_unbind(pn_transport_t *transport);
  *
  * @param[in] transport a transport object
  * @param[in] trace the trace flags
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN void pn_transport_trace(pn_transport_t *transport, pn_trace_t trace);
 
@@ -246,18 +255,24 @@ PN_EXTERN void pn_transport_trace(pn_transport_t *transport, pn_trace_t trace);
  *
  * @param[in] transport a transport object
  * @param[in] tracer the tracing function
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN void pn_transport_set_tracer(pn_transport_t *transport, pn_tracer_t tracer);
 
 /**
- * Get the tracning function used by a transport.
+ * Get the tracing function used by a transport.
  *
  * @param[in] transport a transport object
  * @return the tracing function used by a transport
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN pn_tracer_t pn_transport_get_tracer(pn_transport_t *transport);
 
 /**
+ * **Deprecated** - Use ::pn_transport_attachments().
+ *
  * Get the application context that is associated with a transport object.
  *
  * The application context for a transport may be set using
@@ -269,7 +284,8 @@ PN_EXTERN pn_tracer_t pn_transport_get_tracer(pn_transport_t *transport);
 PN_EXTERN void *pn_transport_get_context(pn_transport_t *transport);
 
 /**
- * @deprecated
+ * **Deprecated** - Use ::pn_transport_attachments().
+ *
  * Set a new application context for a transport object.
  *
  * The application context for a transport object may be retrieved using
@@ -296,6 +312,8 @@ PN_EXTERN pn_record_t *pn_transport_attachments(pn_transport_t *transport);
  *
  * @param[in] transport a transport object
  * @param[in] message the message to be logged
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN void pn_transport_log(pn_transport_t *transport, const char *message);
 
@@ -309,6 +327,8 @@ PN_EXTERN void pn_transport_log(pn_transport_t *transport, const char *message);
  * @param[in] transport a transport object
  * @param[in] fmt the printf formatted message to be logged
  * @param[in] ap a vector containing the format arguments
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN void pn_transport_vlogf(pn_transport_t *transport, const char *fmt, va_list ap);
 
@@ -321,29 +341,33 @@ PN_EXTERN void pn_transport_vlogf(pn_transport_t *transport, const char *fmt, va
  *
  * @param[in] transport a transport object
  * @param[in] fmt the printf formatted message to be logged
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN void pn_transport_logf(pn_transport_t *transport, const char *fmt, ...);
 
 /**
  * Get the maximum allowed channel for a transport.
- * This will be the minimum of 
+ * This will be the minimum of
  *   1. limit imposed by this proton implementation
  *   2. limit imposed by remote peer
  *   3. limit imposed by this application, using pn_transport_set_channel_max()
  *
  * @param[in] transport a transport object
  * @return the maximum allowed channel
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN uint16_t pn_transport_get_channel_max(pn_transport_t *transport);
 
 /**
  * Set the maximum allowed channel number for a transport.
- * Note that this is the maximum channel number allowed, giving a 
- * valid channel number range of [0..channel_max]. Therefore the 
- * maximum number of simultaineously active channels will be 
+ * Note that this is the maximum channel number allowed, giving a
+ * valid channel number range of [0..channel_max]. Therefore the
+ * maximum number of simultaneously active channels will be
  * channel_max plus 1.
  * You can call this function more than once to raise and lower
- * the limit your application imposes on max channels for this 
+ * the limit your application imposes on max channels for this
  * transport.  However, smaller limits may be imposed by this
  * library, or by the remote peer.
  * After the OPEN frame has been sent to the remote peer,
@@ -352,6 +376,8 @@ PN_EXTERN uint16_t pn_transport_get_channel_max(pn_transport_t *transport);
  * @param[in] transport a transport object
  * @param[in] channel_max the maximum allowed channel
  * @return PN_OK, or PN_STATE_ERR if it is too late to change channel_max
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN int pn_transport_set_channel_max(pn_transport_t *transport, uint16_t channel_max);
 
@@ -360,6 +386,8 @@ PN_EXTERN int pn_transport_set_channel_max(pn_transport_t *transport, uint16_t c
  *
  * @param[in] transport a transport object
  * @return the maximum allowed channel of the transport's remote peer
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN uint16_t pn_transport_remote_channel_max(pn_transport_t *transport);
 
@@ -368,6 +396,8 @@ PN_EXTERN uint16_t pn_transport_remote_channel_max(pn_transport_t *transport);
  *
  * @param[in] transport a transport object
  * @return the maximum frame size of the transport object
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN uint32_t pn_transport_get_max_frame(pn_transport_t *transport);
 
@@ -376,6 +406,8 @@ PN_EXTERN uint32_t pn_transport_get_max_frame(pn_transport_t *transport);
  *
  * @param[in] transport a transport object
  * @param[in] size the maximum frame size for the transport object
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN void pn_transport_set_max_frame(pn_transport_t *transport, uint32_t size);
 
@@ -384,6 +416,8 @@ PN_EXTERN void pn_transport_set_max_frame(pn_transport_t *transport, uint32_t si
  *
  * @param[in] transport a transport object
  * @return the maximum frame size of the transport's remote peer
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN uint32_t pn_transport_get_remote_max_frame(pn_transport_t *transport);
 
@@ -394,6 +428,8 @@ PN_EXTERN uint32_t pn_transport_get_remote_max_frame(pn_transport_t *transport);
  *
  * @param[in] transport a transport object
  * @return the transport's idle timeout
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN pn_millis_t pn_transport_get_idle_timeout(pn_transport_t *transport);
 
@@ -404,6 +440,8 @@ PN_EXTERN pn_millis_t pn_transport_get_idle_timeout(pn_transport_t *transport);
  *
  * @param[in] transport a transport object
  * @param[in] timeout the idle timeout for the transport object
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN void pn_transport_set_idle_timeout(pn_transport_t *transport, pn_millis_t timeout);
 
@@ -414,15 +452,18 @@ PN_EXTERN void pn_transport_set_idle_timeout(pn_transport_t *transport, pn_milli
  *
  * @param[in] transport a transport object
  * @return the idle timeout for the transport's remote peer
+ *
+ * @internal XXX Deprecate!
  */
 PN_EXTERN pn_millis_t pn_transport_get_remote_idle_timeout(pn_transport_t *transport);
 
 /**
- * @deprecated
+ * **Deprecated**
  */
 PN_EXTERN ssize_t pn_transport_input(pn_transport_t *transport, const char *bytes, size_t available);
+
 /**
- * @deprecated
+ * **Deprecated**
  */
 PN_EXTERN ssize_t pn_transport_output(pn_transport_t *transport, char *bytes, size_t size);
 
@@ -433,7 +474,7 @@ PN_EXTERN ssize_t pn_transport_output(pn_transport_t *transport, char *bytes, si
  * If the engine is in an exceptional state such as encountering an
  * error condition or reaching the end of stream state, a negative
  * value will be returned indicating the condition. If an error is
- * indicated, futher details can be obtained from
+ * indicated, further details can be obtained from
  * ::pn_transport_error. Calls to ::pn_transport_process may alter the
  * value of this pointer. See ::pn_transport_process for details.
  *
@@ -446,7 +487,7 @@ PN_EXTERN ssize_t pn_transport_capacity(pn_transport_t *transport);
  * Get the transport's tail pointer.
  *
  * The amount of free space following this pointer is reported by
- * ::pn_transport_capacity. Calls to ::pn_transport_process may alther
+ * ::pn_transport_capacity. Calls to ::pn_transport_process may alter
  * the value of this pointer. See ::pn_transport_process for details.
  *
  * @param[in] transport the transport
@@ -457,10 +498,10 @@ PN_EXTERN char *pn_transport_tail(pn_transport_t *transport);
 /**
  * Pushes the supplied bytes into the tail of the transport.
  *
- * This is equivalent to copying @c size bytes afther the tail pointer
+ * This is equivalent to copying @c size bytes after the tail pointer
  * and then calling ::pn_transport_process with an argument of @c
  * size. Only some of the bytes will be copied if there is
- * insufficienty capacity available. Use ::pn_transport_capacity to
+ * insufficient capacity available. Use ::pn_transport_capacity to
  * determine how much capacity the transport has.
  *
  * @param[in] transport the transport
@@ -572,14 +613,17 @@ PN_EXTERN int pn_transport_close_head(pn_transport_t *transport);
 PN_EXTERN bool pn_transport_quiesced(pn_transport_t *transport);
 
 /**
- * Check if a transport is closed.
- *
- * A transport is defined to be closed when both the tail and the head
- * are closed. In other words, when both ::pn_transport_capacity() < 0
- * and ::pn_transport_pending() < 0.
- *
- * @param[in] transport a transport object
- * @return true if the transport is closed, false otherwise
+ * True if pn_transport_close_head() has been called.
+ */
+PN_EXTERN bool pn_transport_head_closed(pn_transport_t *transport);
+
+/**
+ * True if pn_transport_close_tail() has been called.
+ */
+PN_EXTERN bool pn_transport_tail_closed(pn_transport_t *transport);
+
+/**
+ * Equivalent to pn_transport_head_closed(transport) && pn_transport_tail_closed(transport)
  */
 PN_EXTERN bool pn_transport_closed(pn_transport_t *transport);
 
@@ -616,7 +660,8 @@ PN_EXTERN uint64_t pn_transport_get_frames_output(const pn_transport_t *transpor
  */
 PN_EXTERN uint64_t pn_transport_get_frames_input(const pn_transport_t *transport);
 
-/** Access the AMQP Connection associated with the transport.
+/**
+ * Access the AMQP Connection associated with the transport.
  *
  * @param[in] transport a transport object
  * @return the connection context for the transport, or NULL if
@@ -628,7 +673,8 @@ PN_EXTERN pn_connection_t *pn_transport_connection(pn_transport_t *transport);
 }
 #endif
 
-/** @}
+/**
+ * @}
  */
 
 #endif /* transport.h */
