@@ -19,23 +19,24 @@
  *
  */
 
+#include "io.h"
+#include "reactor.h"
+#include "selectable.h"
+#include "platform/platform.h" // pn_i_now
+
 #include <proton/object.h>
 #include <proton/handlers.h>
-#include <proton/io.h>
 #include <proton/event.h>
 #include <proton/transport.h>
 #include <proton/connection.h>
 #include <proton/session.h>
 #include <proton/link.h>
 #include <proton/delivery.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
 
-#include "reactor.h"
-#include "selectable.h"
-#include "platform.h"
-
 struct pn_reactor_t {
   pn_record_t *attachments;
   pn_io_t *io;
@@ -164,11 +165,16 @@ void pn_reactor_set_handler(pn_reactor_t *reactor, pn_handler_t *handler) {
   pn_incref(reactor->handler);
 }
 
-pn_io_t *pn_reactor_io(pn_reactor_t *reactor) {
+pn_io_t *pni_reactor_io(pn_reactor_t *reactor) {
   assert(reactor);
   return reactor->io;
 }
 
+pn_error_t *pn_reactor_error(pn_reactor_t *reactor) {
+  assert(reactor);
+  return pn_io_error(reactor->io);
+}
+
 pn_collector_t *pn_reactor_collector(pn_reactor_t *reactor) {
   assert(reactor);
   return reactor->collector;
@@ -384,6 +390,16 @@ bool pn_reactor_quiesced(pn_reactor_t *reactor) {
   return pn_event_type(event) == PN_REACTOR_QUIESCED;
 }
 
+pn_handler_t *pn_event_root(pn_event_t *event)
+{
+  pn_handler_t *h = pn_record_get_handler(pn_event_attachments(event));
+  return h;
+}
+
+static void pni_event_set_root(pn_event_t *event, pn_handler_t *handler) {
+  pn_record_set_handler(pn_event_attachments(event), handler);
+}
+
 bool pn_reactor_process(pn_reactor_t *reactor) {
   assert(reactor);
   pn_reactor_mark(reactor);
@@ -420,7 +436,8 @@ bool pn_reactor_process(pn_reactor_t *reactor) {
         pn_reactor_update(reactor, reactor->selectable);
         reactor->selectable = NULL;
       } else {
-        pn_collector_put(reactor->collector, PN_OBJECT, reactor, PN_REACTOR_FINAL);
+        if (reactor->previous != PN_REACTOR_FINAL)
+          pn_collector_put(reactor->collector, PN_OBJECT, reactor, PN_REACTOR_FINAL);
         return false;
       }
     }
