@@ -19,11 +19,12 @@
  *
  */
 
+#include "platform/platform.h"
+#include "core/util.h"
+#include "core/engine-internal.h"
+
 #include <proton/ssl.h>
 #include <proton/engine.h>
-#include "engine/engine-internal.h"
-#include "platform.h"
-#include "util.h"
 
 // openssl on windows expects the user to have already included
 // winsock.h
@@ -50,18 +51,16 @@
 #include <fcntl.h>
 #include <assert.h>
 
-
 /** @file
  * SSL/TLS support API.
  *
  * This file contains an OpenSSL-based implemention of the SSL/TLS API.
  */
 
-static int ssl_initialized;
-static int ssl_ex_data_index;
-
 typedef struct pn_ssl_session_t pn_ssl_session_t;
 
+static int ssl_ex_data_index;
+
 struct pn_ssl_domain_t {
 
   SSL_CTX       *ctx;
@@ -71,11 +70,12 @@ struct pn_ssl_domain_t {
   // settings used for all connections
   char *trusted_CAs;
 
-  // session cache
-  pn_ssl_session_t *ssn_cache_head;
-  pn_ssl_session_t *ssn_cache_tail;
+  char *ciphers;
 
   int   ref_count;
+#if OPENSSL_VERSION_NUMBER >= 0x10100000
+  int default_seclevel;
+#endif
   pn_ssl_mode_t mode;
   pn_ssl_verify_mode_t verify_mode;
 
@@ -118,24 +118,16 @@ struct pni_ssl_t {
 
 static inline pn_transport_t *get_transport_internal(pn_ssl_t *ssl)
 {
-    // The external pn_sasl_t is really a pointer to the internal pni_transport_t
-    return ((pn_transport_t *)ssl);
+  // The external pn_sasl_t is really a pointer to the internal pni_transport_t
+  return ((pn_transport_t *)ssl);
 }
 
 static inline pni_ssl_t *get_ssl_internal(pn_ssl_t *ssl)
 {
-    // The external pn_sasl_t is really a pointer to the internal pni_transport_t
-    return ssl ? ((pn_transport_t *)ssl)->ssl : NULL;
+  // The external pn_sasl_t is really a pointer to the internal pni_transport_t
+  return ssl ? ((pn_transport_t *)ssl)->ssl : NULL;
 }
 
-struct pn_ssl_session_t {
-  const char       *id;
-  SSL_SESSION      *session;
-  pn_ssl_session_t *ssn_cache_next;
-  pn_ssl_session_t *ssn_cache_prev;
-};
-
-
 // define two sets of allowable ciphers: those that require authentication, and those
 // that do not require authentication (anonymous).  See ciphers(1).
 #define CIPHERS_AUTHENTICATE    "ALL:!aNULL:!eNULL:@STRENGTH"
@@ -150,8 +142,6 @@ static ssize_t process_input_done(pn_transport_t *transport, unsigned int layer,
 static ssize_t process_output_done(pn_transport_t *transport, unsigned int layer, char *input_data, size_t len);
 static int init_ssl_socket(pn_transport_t *, pni_ssl_t *);
 static void release_ssl_socket( pni_ssl_t * );
-static pn_ssl_session_t *ssn_cache_find( pn_ssl_domain_t *, const char * );
-static void ssl_session_free( pn_ssl_session_t *);
 static size_t buffered_output( pn_transport_t *transport );
 static X509 *get_peer_certificate(pni_ssl_t *ssl);
 
@@ -217,7 +207,7 @@ static int ssl_failed(pn_transport_t *transport)
   // fake a shutdown so the i/o processing code will close properly
   SSL_set_shutdown(ssl->ssl, SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
   // try to grab the first SSL error to add to the failure log
-  char buf[128] = "Unknown error.";
+  char buf[256] = "Unknown error";
   unsigned long ssl_err = ERR_get_error();
   if (ssl_err) {
     ERR_error_string_n( ssl_err, buf, sizeof(buf) );
@@ -274,8 +264,8 @@ static bool match_dns_pattern( const char *hostname,
       int suffix_len = strlen(suffix);
       if (prefix_len && pn_strncasecmp( prefix, slabel, prefix_len )) return false;
       if (suffix_len && pn_strncasecmp( suffix,
-                                     slabel + (strlen(slabel) - suffix_len),
-                                     suffix_len )) return false;
+                                        slabel + (strlen(slabel) - suffix_len),
+                                        suffix_len )) return false;
     }
   }
 
@@ -357,7 +347,7 @@ static int verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 
   if (!matched) {
     ssl_log(transport, "Error: no name matching %s found in peer cert - rejecting handshake.",
-          ssl->peer_hostname);
+            ssl->peer_hostname);
     preverify_ok = 0;
 #ifdef X509_V_ERR_APPLICATION_VERIFICATION
     X509_STORE_CTX_set_error( ctx, X509_V_ERR_APPLICATION_VERIFICATION );
@@ -368,12 +358,22 @@ static int verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
   return preverify_ok;
 }
 
+// This was introduced in v1.1
+#if OPENSSL_VERSION_NUMBER < 0x10100000
+int DH_set0_pqg(DH *dh, BIGNUM *p, BIGNUM *q, BIGNUM *g)
+{
+  dh->p = p;
+  dh->q = q;
+  dh->g = g;
+  return 1;
+}
+#endif
 
 // this code was generated using the command:
 // "openssl dhparam -C -2 2048"
 static DH *get_dh2048(void)
 {
-  static const unsigned char dh2048_p[]={
+  static const unsigned char dhp_2048[]={
     0xAE,0xF7,0xE9,0x66,0x26,0x7A,0xAC,0x0A,0x6F,0x1E,0xCD,0x81,
     0xBD,0x0A,0x10,0x7E,0xFA,0x2C,0xF5,0x2D,0x98,0xD4,0xE7,0xD9,
     0xE4,0x04,0x8B,0x06,0x85,0xF2,0x0B,0xA3,0x90,0x15,0x56,0x0C,
@@ -397,52 +397,77 @@ static DH *get_dh2048(void)
     0xA4,0xED,0xFD,0x49,0x0B,0xE3,0x4A,0xF6,0x28,0xB3,0x98,0xB0,
     0x23,0x1C,0x09,0x33,
   };
-  static const unsigned char dh2048_g[]={
+  static const unsigned char dhg_2048[]={
     0x02,
   };
-  DH *dh;
+  DH *dh = DH_new();
+  BIGNUM *dhp_bn, *dhg_bn;
 
-  if ((dh=DH_new()) == NULL) return(NULL);
-  dh->p=BN_bin2bn(dh2048_p,sizeof(dh2048_p),NULL);
-  dh->g=BN_bin2bn(dh2048_g,sizeof(dh2048_g),NULL);
-  if ((dh->p == NULL) || (dh->g == NULL))
-    { DH_free(dh); return(NULL); }
-  return(dh);
-}
-
-static pn_ssl_session_t *ssn_cache_find( pn_ssl_domain_t *domain, const char *id )
-{
-  pn_timestamp_t now_msec = pn_i_now();
-  long now_sec = (long)(now_msec / 1000);
-  pn_ssl_session_t *ssn = LL_HEAD( domain, ssn_cache );
-  while (ssn) {
-    long expire = SSL_SESSION_get_time( ssn->session )
-      + SSL_SESSION_get_timeout( ssn->session );
-    if (expire < now_sec) {
-      pn_ssl_session_t *next = ssn->ssn_cache_next;
-      LL_REMOVE( domain, ssn_cache, ssn );
-      ssl_session_free( ssn );
-      ssn = next;
-      continue;
-    }
-
-    if (!strcmp(ssn->id, id)) {
-      break;
-    }
-    ssn = ssn->ssn_cache_next;
+  if (dh == NULL)
+    return NULL;
+  dhp_bn = BN_bin2bn(dhp_2048, sizeof (dhp_2048), NULL);
+  dhg_bn = BN_bin2bn(dhg_2048, sizeof (dhg_2048), NULL);
+  if (dhp_bn == NULL || dhg_bn == NULL
+      || !DH_set0_pqg(dh, dhp_bn, NULL, dhg_bn)) {
+    DH_free(dh);
+    BN_free(dhp_bn);
+    BN_free(dhg_bn);
+    return NULL;
   }
-  return ssn;
+  return dh;
 }
 
-static void ssl_session_free( pn_ssl_session_t *ssn)
-{
-  if (ssn) {
-    if (ssn->id) free( (void *)ssn->id );
-    if (ssn->session) SSL_SESSION_free( ssn->session );
-    free( ssn );
+typedef struct {
+  char *id;
+  SSL_SESSION *session;
+} ssl_cache_data;
+
+#define SSL_CACHE_SIZE 4
+static int ssl_cache_ptr = 0;
+static ssl_cache_data ssl_cache[SSL_CACHE_SIZE];
+
+static void ssn_init(void) {
+  ssl_cache_data s = {NULL, NULL};
+  for (int i=0; i<SSL_CACHE_SIZE; i++) {
+    ssl_cache[i] = s;
   }
 }
 
+static void ssn_restore(pn_transport_t *transport, pni_ssl_t *ssl) {
+  if (!ssl->session_id) return;
+  for (int i = ssl_cache_ptr;;) {
+    i = (i==0) ? SSL_CACHE_SIZE-1 : i-1;
+    if (ssl_cache[i].id == NULL) return;
+    if (strcmp(ssl_cache[i].id, ssl->session_id) == 0) {
+      ssl_log( transport, "Restoring previous session id=%s", ssl->session_id );
+      int rc = SSL_set_session( ssl->ssl, ssl_cache[i].session );
+      if (rc != 1) {
+        ssl_log( transport, "Session restore failed, id=%s", ssl->session_id );
+      }
+      return;
+    }
+    if (i == ssl_cache_ptr) return;
+  }
+}
+
+static void ssn_save(pn_transport_t *transport, pni_ssl_t *ssl) {
+  if (ssl->session_id) {
+    // Attach the session id to the session before we close the connection
+    // So that if we find it in the cache later we can figure out the session id
+    SSL_SESSION *session = SSL_get1_session( ssl->ssl );
+    if (session) {
+      ssl_log(transport, "Saving SSL session as %s", ssl->session_id );
+      // If we're overwriting a value, need to free it
+      free(ssl_cache[ssl_cache_ptr].id);
+      if (ssl_cache[ssl_cache_ptr].session) SSL_SESSION_free(ssl_cache[ssl_cache_ptr].session);
+
+      char *id = pn_strdup( ssl->session_id );
+      ssl_cache_data s = {id, session};
+      ssl_cache[ssl_cache_ptr++] = s;
+      if (ssl_cache_ptr==SSL_CACHE_SIZE) ssl_cache_ptr = 0;
+    }
+  }
+}
 
 /** Public API - visible to application code */
 
@@ -451,17 +476,14 @@ bool pn_ssl_present(void)
   return true;
 }
 
+static bool ensure_initialized(void);
+
 pn_ssl_domain_t *pn_ssl_domain( pn_ssl_mode_t mode )
 {
-  if (!ssl_initialized) {
-    ssl_initialized = 1;
-    SSL_library_init();
-    SSL_load_error_strings();
-    OpenSSL_add_all_algorithms();
-    ssl_ex_data_index = SSL_get_ex_new_index( 0, (void *) "org.apache.qpid.proton.ssl",
-                                              NULL, NULL, NULL);
+  if (!ensure_initialized()) {
+      ssl_log_error("Unable to initialize OpenSSL library");
+      return NULL;
   }
-
   pn_ssl_domain_t *domain = (pn_ssl_domain_t *) calloc(1, sizeof(pn_ssl_domain_t));
   if (!domain) return NULL;
 
@@ -472,8 +494,9 @@ pn_ssl_domain_t *pn_ssl_domain( pn_ssl_mode_t mode )
   // known vulnerable ones.  This should allow us to use the latest version
   // of the TLS standard that the installed library supports.
   switch(mode) {
-  case PN_SSL_MODE_CLIENT:
+   case PN_SSL_MODE_CLIENT:
     domain->ctx = SSL_CTX_new(SSLv23_client_method()); // and TLSv1+
+    SSL_CTX_set_session_cache_mode(domain->ctx, SSL_SESS_CACHE_CLIENT);
     if (!domain->ctx) {
       ssl_log_error("Unable to initialize OpenSSL context.");
       free(domain);
@@ -481,7 +504,7 @@ pn_ssl_domain_t *pn_ssl_domain( pn_ssl_mode_t mode )
     }
     break;
 
-  case PN_SSL_MODE_SERVER:
+   case PN_SSL_MODE_SERVER:
     domain->ctx = SSL_CTX_new(SSLv23_server_method()); // and TLSv1+
     if (!domain->ctx) {
       ssl_log_error("Unable to initialize OpenSSL context.");
@@ -490,7 +513,7 @@ pn_ssl_domain_t *pn_ssl_domain( pn_ssl_mode_t mode )
     }
     break;
 
-  default:
+   default:
     pn_transport_logf(NULL, "Invalid value for pn_ssl_mode_t: %d", mode);
     free(domain);
     return NULL;
@@ -501,6 +524,9 @@ pn_ssl_domain_t *pn_ssl_domain( pn_ssl_mode_t mode )
   // Mitigate the CRIME vulnerability
   SSL_CTX_set_options(domain->ctx, SSL_OP_NO_COMPRESSION);
 #endif
+#if OPENSSL_VERSION_NUMBER >= 0x10100000
+  domain->default_seclevel = SSL_CTX_get_security_level(domain->ctx);
+#endif
 
   // by default, allow anonymous ciphers so certificates are not required 'out of the box'
   if (!SSL_CTX_set_cipher_list( domain->ctx, CIPHERS_ANONYMOUS )) {
@@ -529,26 +555,19 @@ void pn_ssl_domain_free( pn_ssl_domain_t *domain )
 {
   if (--domain->ref_count == 0) {
 
-    pn_ssl_session_t *ssn = LL_HEAD( domain, ssn_cache );
-    while (ssn) {
-      pn_ssl_session_t *next = ssn->ssn_cache_next;
-      LL_REMOVE( domain, ssn_cache, ssn );
-      ssl_session_free( ssn );
-      ssn = next;
-    }
-
     if (domain->ctx) SSL_CTX_free(domain->ctx);
     if (domain->keyfile_pw) free(domain->keyfile_pw);
     if (domain->trusted_CAs) free(domain->trusted_CAs);
+    if (domain->ciphers) free(domain->ciphers);
     free(domain);
   }
 }
 
 
 int pn_ssl_domain_set_credentials( pn_ssl_domain_t *domain,
-                               const char *certificate_file,
-                               const char *private_key_file,
-                               const char *password)
+                                   const char *certificate_file,
+                                   const char *private_key_file,
+                                   const char *password)
 {
   if (!domain || !domain->ctx) return -1;
 
@@ -570,7 +589,7 @@ int pn_ssl_domain_set_credentials( pn_ssl_domain_t *domain,
 
   if (SSL_CTX_check_private_key(domain->ctx) != 1) {
     ssl_log_error("The key file %s is not consistent with the certificate %s",
-                   private_key_file, certificate_file);
+                  private_key_file, certificate_file);
     return -5;
   }
 
@@ -579,14 +598,77 @@ int pn_ssl_domain_set_credentials( pn_ssl_domain_t *domain,
   // bug in older versions of OpenSSL: servers may request client cert even if anonymous
   // cipher was negotiated.  TLSv1 will reject such a request.  Hack: once a cert is
   // configured, allow only authenticated ciphers.
-  if (!SSL_CTX_set_cipher_list( domain->ctx, CIPHERS_AUTHENTICATE )) {
-      ssl_log_error("Failed to set cipher list to %s", CIPHERS_AUTHENTICATE);
-      return -6;
+  if (!domain->ciphers && !SSL_CTX_set_cipher_list( domain->ctx, CIPHERS_AUTHENTICATE )) {
+    ssl_log_error("Failed to set cipher list to %s", CIPHERS_AUTHENTICATE);
+    return -6;
   }
 
   return 0;
 }
 
+int pn_ssl_domain_set_ciphers(pn_ssl_domain_t *domain, const char *ciphers)
+{
+  if (!SSL_CTX_set_cipher_list(domain->ctx, ciphers)) {
+    ssl_log_error("Failed to set cipher list to %s", ciphers);
+    return -6;
+  }
+  if (domain->ciphers) free(domain->ciphers);
+  domain->ciphers = pn_strdup(ciphers);
+  return 0;
+}
+
+int pn_ssl_domain_set_protocols(pn_ssl_domain_t *domain, const char *protocols)
+{
+  static const struct {
+    const char *name;
+    const long option;
+  } protocol_options[] =
+  {
+    {"TLSv1",   SSL_OP_NO_TLSv1},
+    {"TLSv1.1", SSL_OP_NO_TLSv1_1},
+    {"TLSv1.2", SSL_OP_NO_TLSv1_2}
+  };
+  static const char seps[]    = " ,;";
+  static const long all_prots = SSL_OP_NO_TLSv1 | SSL_OP_NO_TLSv1_1 | SSL_OP_NO_TLSv1_2;
+
+  // Start with all protocols turned off
+  long options = all_prots;
+
+  // For each separate token in protocols
+  const char *token = protocols;
+  while (*token!=0) {
+    // Find next separator
+    size_t tsize = strcspn(token, seps);
+    while (tsize==0 && *token!=0) {
+      ++token;
+      tsize = strcspn(token, seps);
+    }
+    if (tsize==0) break; // No more tokens
+
+    // Linear search the posibilities for the option to set
+    for (size_t i = 0; i<sizeof(protocol_options)/sizeof(*protocol_options); ++i) {
+      if (strncmp(token, protocol_options[i].name, tsize)==0) {
+        options &= ~protocol_options[i].option;
+        goto found;
+      }
+    }
+    // Didn't find any match - error
+    return PN_ARG_ERR;
+
+found:
+    token += tsize;
+  }
+
+  // Check if we found anything
+  if (options==all_prots) return PN_ARG_ERR;
+
+  // Debug test only
+  //printf("%30s %016lx (~%016lx)\n", protocols, options, ~options&all_prots);
+
+  SSL_CTX_clear_options(domain->ctx, all_prots);
+  SSL_CTX_set_options(domain->ctx, options);
+  return 0;
+}
 
 int pn_ssl_domain_set_trusted_ca_db(pn_ssl_domain_t *domain,
                                     const char *certificate_db)
@@ -629,12 +711,16 @@ int pn_ssl_domain_set_peer_authentication(pn_ssl_domain_t *domain,
   if (!domain) return -1;
 
   switch (mode) {
-  case PN_SSL_VERIFY_PEER:
-  case PN_SSL_VERIFY_PEER_NAME:
+   case PN_SSL_VERIFY_PEER:
+   case PN_SSL_VERIFY_PEER_NAME:
+
+#if OPENSSL_VERSION_NUMBER >= 0x10100000
+    SSL_CTX_set_security_level(domain->ctx, domain->default_seclevel);
+#endif
 
     if (!domain->has_ca_db) {
       pn_transport_logf(NULL, "Error: cannot verify peer without a trusted CA configured.\n"
-                 "       Use pn_ssl_domain_set_trusted_ca_db()");
+                        "       Use pn_ssl_domain_set_trusted_ca_db()");
       return -1;
     }
 
@@ -647,7 +733,7 @@ int pn_ssl_domain_set_peer_authentication(pn_ssl_domain_t *domain,
       }
       if (!domain->has_certificate) {
         pn_transport_logf(NULL, "Error: Server cannot verify peer without configuring a certificate.\n"
-                   "       Use pn_ssl_domain_set_credentials()");
+                          "       Use pn_ssl_domain_set_credentials()");
       }
 
       if (domain->trusted_CAs) free(domain->trusted_CAs);
@@ -669,11 +755,15 @@ int pn_ssl_domain_set_peer_authentication(pn_ssl_domain_t *domain,
 #endif
     break;
 
-  case PN_SSL_ANONYMOUS_PEER:   // hippie free love mode... :)
+   case PN_SSL_ANONYMOUS_PEER:   // hippie free love mode... :)
+#if OPENSSL_VERSION_NUMBER >= 0x10100000
+    // Must use lowest OpenSSL security level to enable anonymous ciphers.
+    SSL_CTX_set_security_level(domain->ctx, 0);
+#endif
     SSL_CTX_set_verify( domain->ctx, SSL_VERIFY_NONE, NULL );
     break;
 
-  default:
+   default:
     pn_transport_logf(NULL, "Invalid peer authentication mode given." );
     return -1;
   }
@@ -683,35 +773,35 @@ int pn_ssl_domain_set_peer_authentication(pn_ssl_domain_t *domain,
 }
 
 const pn_io_layer_t ssl_layer = {
-    process_input_ssl,
-    process_output_ssl,
-    handle_error_ssl,
-    NULL,
-    buffered_output
+  process_input_ssl,
+  process_output_ssl,
+  handle_error_ssl,
+  NULL,
+  buffered_output
 };
 
 const pn_io_layer_t ssl_input_closed_layer = {
-    process_input_done,
-    process_output_ssl,
-    handle_error_ssl,
-    NULL,
-    buffered_output
+  process_input_done,
+  process_output_ssl,
+  handle_error_ssl,
+  NULL,
+  buffered_output
 };
 
 const pn_io_layer_t ssl_output_closed_layer = {
-    process_input_ssl,
-    process_output_done,
-    handle_error_ssl,
-    NULL,
-    buffered_output
+  process_input_ssl,
+  process_output_done,
+  handle_error_ssl,
+  NULL,
+  buffered_output
 };
 
 const pn_io_layer_t ssl_closed_layer = {
-    process_input_done,
-    process_output_done,
-    handle_error_ssl,
-    NULL,
-    buffered_output
+  process_input_done,
+  process_output_done,
+  handle_error_ssl,
+  NULL,
+  buffered_output
 };
 
 int pn_ssl_init(pn_ssl_t *ssl0, pn_ssl_domain_t *domain, const char *session_id)
@@ -760,11 +850,11 @@ bool pn_ssl_get_cipher_name(pn_ssl_t *ssl0, char *buffer, size_t size )
   const SSL_CIPHER *c;
 
   pni_ssl_t *ssl = get_ssl_internal(ssl0);
-  *buffer = '\0';
+  if (buffer && size) *buffer = '\0';
   if (ssl->ssl && (c = SSL_get_current_cipher( ssl->ssl ))) {
     const char *v = SSL_CIPHER_get_name(c);
-    if (v) {
-      snprintf( buffer, size, "%s", v );
+    if (buffer && v) {
+      pni_snprintf( buffer, size, "%s", v );
       return true;
     }
   }
@@ -776,11 +866,11 @@ bool pn_ssl_get_protocol_name(pn_ssl_t *ssl0, char *buffer, size_t size )
   const SSL_CIPHER *c;
 
   pni_ssl_t *ssl = get_ssl_internal(ssl0);
-  *buffer = '\0';
+  if (buffer && size) *buffer = '\0';
   if (ssl->ssl && (c = SSL_get_current_cipher( ssl->ssl ))) {
     const char *v = SSL_CIPHER_get_version(c);
-    if (v) {
-      snprintf( buffer, size, "%s", v );
+    if (buffer && v) {
+      pni_snprintf( buffer, size, "%s", v );
       return true;
     }
   }
@@ -843,9 +933,9 @@ pn_ssl_t *pn_ssl(pn_transport_t *transport)
 
 static int keyfile_pw_cb(char *buf, int size, int rwflag, void *userdata)
 {
-    strncpy(buf, (char *)userdata, size);   // @todo: un-obfuscate me!!!
-    buf[size - 1] = '\0';
-    return(strlen(buf));
+  strncpy(buf, (char *)userdata, size);   // @todo: un-obfuscate me!!!
+  buf[size - 1] = '\0';
+  return(strlen(buf));
 }
 
 
@@ -854,20 +944,7 @@ static int start_ssl_shutdown(pn_transport_t *transport)
   pni_ssl_t *ssl = transport->ssl;
   if (!ssl->ssl_shutdown) {
     ssl_log(transport, "Shutting down SSL connection...");
-    if (ssl->session_id) {
-      // save the negotiated credentials before we close the connection
-      pn_ssl_session_t *ssn = (pn_ssl_session_t *)calloc( 1, sizeof(pn_ssl_session_t));
-      if (ssn) {
-        ssn->id = pn_strdup( ssl->session_id );
-        ssn->session = SSL_get1_session( ssl->ssl );
-        if (ssn->session) {
-          ssl_log(transport, "Saving SSL session as %s", ssl->session_id );
-          LL_ADD( ssl->domain, ssn_cache, ssn );
-        } else {
-          ssl_session_free( ssn );
-        }
-      }
-    }
+    ssn_save(transport, ssl);
     ssl->ssl_shutdown = true;
     BIO_ssl_shutdown( ssl->bio_ssl );
   }
@@ -893,6 +970,7 @@ static ssize_t process_input_ssl( pn_transport_t *transport, unsigned int layer,
 
   do {
     work_pending = false;
+    ERR_clear_error();
 
     // Write to network bio as much as possible, consuming bytes/available
 
@@ -927,13 +1005,13 @@ static ssize_t process_input_ssl( pn_transport_t *transport, unsigned int layer,
         if (!BIO_should_retry(ssl->bio_ssl)) {
           int reason = SSL_get_error( ssl->ssl, read );
           switch (reason) {
-          case SSL_ERROR_ZERO_RETURN:
+           case SSL_ERROR_ZERO_RETURN:
             // SSL closed cleanly
             ssl_log(transport, "SSL connection has closed");
             start_ssl_shutdown(transport);  // KAG: not sure - this may not be necessary
             ssl->ssl_closed = true;
             break;
-          default:
+           default:
             // unexpected error
             return (ssize_t)ssl_failed(transport);
           }
@@ -963,7 +1041,7 @@ static ssize_t process_input_ssl( pn_transport_t *transport, unsigned int layer,
           ssl_log( transport, "Application consumed %d bytes from peer", (int) consumed );
         } else if (consumed < 0) {
           ssl_log(transport, "Application layer closed its input, error=%d (discarding %d bytes)",
-               (int) consumed, (int)ssl->in_count);
+                  (int) consumed, (int)ssl->in_count);
           ssl->in_count = 0;    // discard any pending input
           ssl->app_input_closed = consumed;
           if (ssl->app_output_closed && ssl->out_count == 0) {
@@ -1042,6 +1120,8 @@ static ssize_t process_output_ssl( pn_transport_t *transport, unsigned int layer
 
   do {
     work_pending = false;
+    ERR_clear_error();
+
     // first, get any pending application output, if possible
 
     if (!ssl->app_output_closed && ssl->out_count < ssl->out_size) {
@@ -1053,7 +1133,7 @@ static ssize_t process_output_ssl( pn_transport_t *transport, unsigned int layer
       } else {
         if (app_bytes < 0) {
           ssl_log(transport, "Application layer closed its output, error=%d (%d bytes pending send)",
-               (int) app_bytes, (int) ssl->out_count);
+                  (int) app_bytes, (int) ssl->out_count);
           ssl->app_output_closed = app_bytes;
         }
       }
@@ -1074,14 +1154,14 @@ static ssize_t process_output_ssl( pn_transport_t *transport, unsigned int layer
           if (!BIO_should_retry(ssl->bio_ssl)) {
             int reason = SSL_get_error( ssl->ssl, wrote );
             switch (reason) {
-            case SSL_ERROR_ZERO_RETURN:
+             case SSL_ERROR_ZERO_RETURN:
               // SSL closed cleanly
               ssl_log(transport, "SSL connection has closed");
               start_ssl_shutdown(transport); // KAG: not sure - this may not be necessary
               ssl->out_count = 0;      // can no longer write to socket, so erase app output data
               ssl->ssl_closed = true;
               break;
-            default:
+             default:
               // unexpected error
               return (ssize_t)ssl_failed(transport);
             }
@@ -1168,18 +1248,7 @@ static int init_ssl_socket(pn_transport_t* transport, pni_ssl_t *ssl)
 #endif
 
   // restore session, if available
-  if (ssl->session_id) {
-    pn_ssl_session_t *ssn = ssn_cache_find( ssl->domain, ssl->session_id );
-    if (ssn) {
-      ssl_log( transport, "Restoring previous session id=%s", ssn->id );
-      int rc = SSL_set_session( ssl->ssl, ssn->session );
-      if (rc != 1) {
-        ssl_log( transport, "Session restore failed, id=%s", ssn->id );
-      }
-      LL_REMOVE( ssl->domain, ssn_cache, ssn );
-      ssl_session_free( ssn );
-    }
-  }
+  ssn_restore(transport, ssl);
 
   // now layer a BIO over the SSL socket
   ssl->bio_ssl = BIO_new(BIO_f_ssl());
@@ -1214,7 +1283,7 @@ static void release_ssl_socket(pni_ssl_t *ssl)
 {
   if (ssl->bio_ssl) BIO_free(ssl->bio_ssl);
   if (ssl->ssl) {
-      SSL_free(ssl->ssl);       // will free bio_ssl_io
+    SSL_free(ssl->ssl);       // will free bio_ssl_io
   } else {
     if (ssl->bio_ssl_io) BIO_free(ssl->bio_ssl_io);
   }
@@ -1232,9 +1301,9 @@ pn_ssl_resume_status_t pn_ssl_resume_status(pn_ssl_t *ssl0)
   pni_ssl_t *ssl = get_ssl_internal(ssl0);
   if (!ssl || !ssl->ssl) return PN_SSL_RESUME_UNKNOWN;
   switch (SSL_session_reused( ssl->ssl )) {
-  case 0: return PN_SSL_RESUME_NEW;
-  case 1: return PN_SSL_RESUME_REUSED;
-  default: break;
+   case 0: return PN_SSL_RESUME_NEW;
+   case 1: return PN_SSL_RESUME_REUSED;
+   default: break;
   }
   return PN_SSL_RESUME_UNKNOWN;
 }
@@ -1310,124 +1379,124 @@ const char* pn_ssl_get_remote_subject(pn_ssl_t *ssl0)
 
 int pn_ssl_get_cert_fingerprint(pn_ssl_t *ssl0, char *fingerprint, size_t fingerprint_length, pn_ssl_hash_alg hash_alg)
 {
-    const char *digest_name = NULL;
-    size_t min_required_length;
+  const char *digest_name = NULL;
+  size_t min_required_length;
 
-    // old versions of python expect fingerprint to contain a valid string on
-    // return from this function
-    fingerprint[0] = 0;
+  // old versions of python expect fingerprint to contain a valid string on
+  // return from this function
+  fingerprint[0] = 0;
 
-    // Assign the correct digest_name value based on the enum values.
-    switch (hash_alg) {
-        case PN_SSL_SHA1 :
-            min_required_length = 41; // 40 hex characters + 1 '\0' character
-            digest_name = "sha1";
-            break;
-        case PN_SSL_SHA256 :
-            min_required_length = 65; // 64 hex characters + 1 '\0' character
-            digest_name = "sha256";
-            break;
-        case PN_SSL_SHA512 :
-            min_required_length = 129; // 128 hex characters + 1 '\0' character
-            digest_name = "sha512";
-            break;
-        case PN_SSL_MD5 :
-            min_required_length = 33; // 32 hex characters + 1 '\0' character
-            digest_name = "md5";
-            break;
-        default:
-            ssl_log_error("Unknown or unhandled hash algorithm %i \n", hash_alg);
-            return PN_ERR;
+  // Assign the correct digest_name value based on the enum values.
+  switch (hash_alg) {
+   case PN_SSL_SHA1 :
+    min_required_length = 41; // 40 hex characters + 1 '\0' character
+    digest_name = "sha1";
+    break;
+   case PN_SSL_SHA256 :
+    min_required_length = 65; // 64 hex characters + 1 '\0' character
+    digest_name = "sha256";
+    break;
+   case PN_SSL_SHA512 :
+    min_required_length = 129; // 128 hex characters + 1 '\0' character
+    digest_name = "sha512";
+    break;
+   case PN_SSL_MD5 :
+    min_required_length = 33; // 32 hex characters + 1 '\0' character
+    digest_name = "md5";
+    break;
+   default:
+    ssl_log_error("Unknown or unhandled hash algorithm %i \n", hash_alg);
+    return PN_ERR;
 
+  }
+
+  if(fingerprint_length < min_required_length) {
+    ssl_log_error("Insufficient fingerprint_length %i. fingerprint_length must be %i or above for %s digest\n",
+                  fingerprint_length, min_required_length, digest_name);
+    return PN_ERR;
+  }
+
+  const EVP_MD  *digest = EVP_get_digestbyname(digest_name);
+
+  pni_ssl_t *ssl = get_ssl_internal(ssl0);
+  X509 *cert = get_peer_certificate(ssl);
+
+  if(cert) {
+    unsigned int len;
+
+    unsigned char bytes[64]; // sha512 uses 64 octets, we will use that as the maximum.
+
+    if (X509_digest(cert, digest, bytes, &len) != 1) {
+      ssl_log_error("Failed to extract X509 digest\n");
+      return PN_ERR;
     }
 
-    if(fingerprint_length < min_required_length) {
-        ssl_log_error("Insufficient fingerprint_length %i. fingerprint_length must be %i or above for %s digest\n",
-            fingerprint_length, min_required_length, digest_name);
-        return PN_ERR;
+    char *cursor = fingerprint;
+
+    for (size_t i=0; i<len ; i++) {
+      cursor +=  pni_snprintf((char *)cursor, fingerprint_length, "%02x", bytes[i]);
+      fingerprint_length = fingerprint_length - 2;
     }
 
-    const EVP_MD  *digest = EVP_get_digestbyname(digest_name);
+    return PN_OK;
+  }
+  else {
+    ssl_log_error("No certificate is available yet \n");
+    return PN_ERR;
+  }
 
-    pni_ssl_t *ssl = get_ssl_internal(ssl0);
-
-    X509 *cert = get_peer_certificate(ssl);
-
-    if(cert) {
-        unsigned int len;
-
-        unsigned char bytes[64]; // sha512 uses 64 octets, we will use that as the maximum.
-
-        if (X509_digest(cert, digest, bytes, &len) != 1) {
-            ssl_log_error("Failed to extract X509 digest\n");
-            return PN_ERR;
-       }
-
-        char *cursor = fingerprint;
-
-        for (size_t i=0; i<len ; i++) {
-            cursor +=  snprintf((char *)cursor, fingerprint_length, "%02x", bytes[i]);
-            fingerprint_length = fingerprint_length - 2;
-        }
-
-        return PN_OK;
-    }
-    else {
-        ssl_log_error("No certificate is available yet \n");
-        return PN_ERR;
-    }
-
-    return 0;
+  return 0;
 }
 
 
 const char* pn_ssl_get_remote_subject_subfield(pn_ssl_t *ssl0, pn_ssl_cert_subject_subfield field)
 {
-    int openssl_field = 0;
-
-    // Assign openssl internal representations of field values to openssl_field
-    switch (field) {
-        case PN_SSL_CERT_SUBJECT_COUNTRY_NAME :
-            openssl_field = NID_countryName;
-            break;
-        case PN_SSL_CERT_SUBJECT_STATE_OR_PROVINCE :
-            openssl_field = NID_stateOrProvinceName;
-            break;
-        case PN_SSL_CERT_SUBJECT_CITY_OR_LOCALITY :
-            openssl_field = NID_localityName;
-            break;
-        case PN_SSL_CERT_SUBJECT_ORGANIZATION_NAME :
-            openssl_field = NID_organizationName;
-            break;
-        case PN_SSL_CERT_SUBJECT_ORGANIZATION_UNIT :
-            openssl_field = NID_organizationalUnitName;
-            break;
-        case PN_SSL_CERT_SUBJECT_COMMON_NAME :
-            openssl_field = NID_commonName;
-            break;
-        default:
-            ssl_log_error("Unknown or unhandled certificate subject subfield %i \n", field);
-            return NULL;
-    }
-
-    pni_ssl_t *ssl = get_ssl_internal(ssl0);
-    X509 *cert = get_peer_certificate(ssl);
-
-    X509_NAME *subject_name = X509_get_subject_name(cert);
-
-    // TODO (gmurthy) - A server side cert subject field can have more than one common name like this - Subject: CN=www.domain1.com, CN=www.domain2.com, see https://bugzilla.mozilla.org/show_bug.cgi?id=380656
-    // For now, we will only return the first common name if there is more than one common name in the cert
-    int index = X509_NAME_get_index_by_NID(subject_name, openssl_field, -1);
-
-    if (index > -1) {
-        X509_NAME_ENTRY *ne = X509_NAME_get_entry(subject_name, index);
-        if(ne) {
-            ASN1_STRING *name_asn1 = X509_NAME_ENTRY_get_data(ne);
-            return (char *) name_asn1->data;
-        }
-    }
+  int openssl_field = 0;
 
+  // Assign openssl internal representations of field values to openssl_field
+  switch (field) {
+   case PN_SSL_CERT_SUBJECT_COUNTRY_NAME :
+    openssl_field = NID_countryName;
+    break;
+   case PN_SSL_CERT_SUBJECT_STATE_OR_PROVINCE :
+    openssl_field = NID_stateOrProvinceName;
+    break;
+   case PN_SSL_CERT_SUBJECT_CITY_OR_LOCALITY :
+    openssl_field = NID_localityName;
+    break;
+   case PN_SSL_CERT_SUBJECT_ORGANIZATION_NAME :
+    openssl_field = NID_organizationName;
+    break;
+   case PN_SSL_CERT_SUBJECT_ORGANIZATION_UNIT :
+    openssl_field = NID_organizationalUnitName;
+    break;
+   case PN_SSL_CERT_SUBJECT_COMMON_NAME :
+    openssl_field = NID_commonName;
+    break;
+   default:
+    ssl_log_error("Unknown or unhandled certificate subject subfield %i \n", field);
     return NULL;
+  }
+
+  pni_ssl_t *ssl = get_ssl_internal(ssl0);
+  X509 *cert = get_peer_certificate(ssl);
+  if (!cert) return NULL;
+
+  X509_NAME *subject_name = X509_get_subject_name(cert);
+
+  // TODO (gmurthy) - A server side cert subject field can have more than one common name like this - Subject: CN=www.domain1.com, CN=www.domain2.com, see https://bugzilla.mozilla.org/show_bug.cgi?id=380656
+  // For now, we will only return the first common name if there is more than one common name in the cert
+  int index = X509_NAME_get_index_by_NID(subject_name, openssl_field, -1);
+
+  if (index > -1) {
+    X509_NAME_ENTRY *ne = X509_NAME_get_entry(subject_name, index);
+    if(ne) {
+      ASN1_STRING *name_asn1 = X509_NAME_ENTRY_get_data(ne);
+      return (char *) name_asn1->data;
+    }
+  }
+
+  return NULL;
 }
 
 static ssize_t process_input_done(pn_transport_t *transport, unsigned int layer, const char *input_data, size_t len)
@@ -1452,3 +1521,88 @@ static size_t buffered_output(pn_transport_t *transport)
   }
   return count;
 }
+
+
+/* Thread-safe locking and initialization for POSIX and Windows */
+
+static bool init_ok = false;
+
+#ifdef _WIN32
+
+typedef CRITICAL_SECTION pni_mutex_t;
+static inline void pni_mutex_init(pni_mutex_t *m) { InitializeCriticalSection(m); }
+static inline void pni_mutex_lock(pni_mutex_t *m) { EnterCriticalSection(m); }
+static inline void pni_mutex_unlock(pni_mutex_t *m) { LeaveCriticalSection(m); }
+static inline unsigned long id_callback(void) { return (unsigned long)GetCurrentThreadId(); }
+INIT_ONCE initialize_once = INIT_ONCE_STATIC_INIT;
+static inline bool ensure_initialized(void) {
+  void* dummy;
+  InitOnceExecuteOnce(&initialize_once, &initialize, NULL, &dummy);
+  return init_ok;
+}
+
+#else  /* POSIX */
+
+#include <pthread.h>
+
+static void initialize(void);
+
+typedef pthread_mutex_t pni_mutex_t;
+static inline int pni_mutex_init(pni_mutex_t *m) { return pthread_mutex_init(m, NULL); }
+static inline int pni_mutex_lock(pni_mutex_t *m) { return pthread_mutex_lock(m); }
+static inline int pni_mutex_unlock(pni_mutex_t *m) { return pthread_mutex_unlock(m); }
+static inline unsigned long id_callback(void) { return (unsigned long)pthread_self(); }
+static pthread_once_t initialize_once = PTHREAD_ONCE_INIT;
+static inline bool ensure_initialized(void) {
+  pthread_once(&initialize_once, &initialize);
+  return init_ok;
+}
+
+#endif
+
+static pni_mutex_t *locks = NULL;     /* Lock array for openssl */
+
+/* Callback function for openssl locking */
+static void locking_callback(int mode, int n, const char *file, int line) {
+  if(mode & CRYPTO_LOCK)
+    pni_mutex_lock(&locks[n]);
+  else
+    pni_mutex_unlock(&locks[n]);
+}
+
+static void initialize(void) {
+  int i;
+  SSL_library_init();
+  SSL_load_error_strings();
+  OpenSSL_add_all_algorithms();
+  ssl_ex_data_index = SSL_get_ex_new_index( 0, (void *) "org.apache.qpid.proton.ssl",
+                                            NULL, NULL, NULL);
+  ssn_init();
+  locks = (pni_mutex_t*)malloc(CRYPTO_num_locks() * sizeof(pni_mutex_t));
+  if (!locks) return;
+  for(i = 0;  i < CRYPTO_num_locks();  i++)
+    pni_mutex_init(&locks[i]);
+  CRYPTO_set_id_callback(&id_callback);
+  CRYPTO_set_locking_callback(&locking_callback);
+  /* In recent versions of openssl, the set_callback functions are no-op macros,
+     so we need to take steps to stop the compiler complaining about unused functions. */
+  (void)&id_callback;
+  (void)&locking_callback;
+  init_ok = true;
+}
+
+/* TODO aconway 2017-10-16: There is no opportunity to clean up the locks as proton has no
+   final shut-down call. If it did, we should call this: */
+/*
+static void shutdown(void) {
+  CRYPTO_set_id_callback(NULL);
+  CRYPTO_set_locking_callback(NULL);
+  if(locks)  {
+    int i;
+    for(i = 0;  i < CRYPTO_num_locks();  i++)
+      pni_mutex_destroy(&locks[i]);
+    free(locks);
+    locks = NULL;
+  }
+}
+*/
