@@ -21,6 +21,8 @@
  *
  */
 
+#include "proton/ssl.h"
+
 /** @file
  * Internal API for SSL/TLS support in the Driver Layer.
  *
@@ -29,8 +31,6 @@
  */
 
 // release the SSL context
-void pn_ssl_free( pn_ssl_t *ssl);
-
-void pn_ssl_trace(pn_ssl_t *ssl, pn_trace_t trace);
+void pn_ssl_free(pn_transport_t *transport);
 
 #endif /* ssl-internal.h */
