@@ -35,7 +35,7 @@ if not hasattr(gc, "garbage"):
 # future test areas
 #  + different permutations of setup
 #   - creating deliveries and calling input/output before opening the session/link
-#  + shrinking output_size down to something small? should the enginge buffer?
+#  + shrinking output_size down to something small? should the engine buffer?
 #  + resuming
 #    - locally and remotely created deliveries with the same tag
 
@@ -240,16 +240,10 @@ class ConnectionTest(Test):
     self.c1.transport.channel_max = value
     self.c1.open()
     self.pump()
-    if "java" in sys.platform:
-      assert self.c1.transport.channel_max == 65535, (self.c1.transport.channel_max, value)
-    else:
-      assert self.c1.transport.channel_max == 32767, (self.c1.transport.channel_max, value)
+    assert self.c1.transport.channel_max == 32767, (self.c1.transport.channel_max, value)
 
   def test_channel_max_raise_and_lower(self):
-    if "java" in sys.platform:
-      upper_limit = 65535
-    else:
-      upper_limit = 32767
+    upper_limit = 32767
 
     # It's OK to lower the max below upper_limit.
     self.c1.transport.channel_max = 12345
@@ -274,8 +268,9 @@ class ConnectionTest(Test):
     assert self.c1.transport.channel_max == upper_limit
 
 
+  # TODO: Currently failing test - PROTON-1759 - skip
   def test_channel_max_limits_sessions(self):
-    return
+    raise Skipped('Test fails - PROTON-1759')
     # This is an index -- so max number of channels should be 1.
     self.c1.transport.channel_max = 0
     self.c1.open()
@@ -286,6 +281,8 @@ class ConnectionTest(Test):
     self.pump()
     try:
       ssn_1 = self.c2.session()
+      ssn_1.open()
+      self.pump()
       assert False, "expected session exception"
     except SessionException:
       pass
@@ -308,9 +305,6 @@ class ConnectionTest(Test):
     assert c2.state == Endpoint.LOCAL_ACTIVE | Endpoint.REMOTE_CLOSED
 
   def test_user_config(self):
-    if "java" in sys.platform:
-      raise Skipped("Unsupported API")
-
     self.c1.user = "vindaloo"
     self.c1.password = "secret"
     self.c1.open()
@@ -746,6 +740,15 @@ class LinkTest(Test):
     assert self.snd.remote_rcv_settle_mode == Link.RCV_SECOND
     assert self.rcv.remote_snd_settle_mode == Link.SND_UNSETTLED
 
+  def test_max_message_size(self):
+    assert self.snd.max_message_size == 0
+    assert self.rcv.remote_max_message_size == 0
+    self.snd.max_message_size = 13579
+    self.snd.open()
+    self.rcv.open()
+    self.pump()
+    assert self.rcv.remote_max_message_size == 13579
+
   def test_cleanup(self):
     snd, rcv = self.link("test-link")
     snd.open()
@@ -1170,7 +1173,7 @@ class MaxFrameTransferTest(Test):
 
   def testBigMessage(self):
     """
-    Test transfering a big message.
+    Test transferring a big message.
     """
     self.snd, self.rcv = self.link("test-link")
     self.c1 = self.snd.session.connection
@@ -2019,8 +2022,6 @@ class ServerTest(Test):
   def testKeepalive(self):
     """ Verify that idle frames are sent to keep a Connection alive
     """
-    if "java" in sys.platform:
-      raise Skipped()
     idle_timeout = self.delay
     server = common.TestServer()
     server.start()
@@ -2052,8 +2053,6 @@ class ServerTest(Test):
     """ Verify that a Connection is terminated properly when Idle frames do not
     arrive in a timely manner.
     """
-    if "java" in sys.platform:
-      raise Skipped()
     idle_timeout = self.delay
     server = common.TestServer(idle_timeout=idle_timeout)
     server.start()
@@ -2607,9 +2606,9 @@ class IdleTimeoutEventTest(PeerTest):
     self.transport.idle_timeout = self.delay
     self.connection.open()
     self.half_pump()
-    self.transport.tick(time())
-    sleep(self.delay*2)
-    self.transport.tick(time())
+    t = time()
+    self.transport.tick(t)
+    self.transport.tick(t + self.delay*4)
     self.expect(Event.CONNECTION_INIT, Event.CONNECTION_BOUND,
                 Event.CONNECTION_LOCAL_OPEN, Event.TRANSPORT,
                 Event.TRANSPORT_ERROR, Event.TRANSPORT_TAIL_CLOSED)
@@ -2641,8 +2640,6 @@ class DeliverySegFaultTest(Test):
 class SaslEventTest(CollectorTest):
 
   def testAnonymousNoInitialResponse(self):
-    if "java" in sys.platform:
-      raise Skipped()
     conn = Connection()
     conn.collect(self.collector)
     transport = Transport(Transport.SERVER)
@@ -2660,8 +2657,6 @@ class SaslEventTest(CollectorTest):
     self.expect()
 
   def testPipelinedServerReadFirst(self):
-    if "java" in sys.platform:
-      raise Skipped()
     conn = Connection()
     conn.collect(self.collector)
     transport = Transport(Transport.CLIENT)
@@ -2690,8 +2685,6 @@ class SaslEventTest(CollectorTest):
     assert server.sasl().outcome == SASL.OK
 
   def testPipelinedServerWriteFirst(self):
-    if "java" in sys.platform:
-      raise Skipped()
     conn = Connection()
     conn.collect(self.collector)
     transport = Transport(Transport.CLIENT)
