@@ -25,7 +25,7 @@ from proton._compat import str2bin
 
 def find_test_interop_dir():
     """Walk up the directory tree to find the tests directory."""
-    f = os.path.dirname(__file__)
+    f = os.path.dirname(os.path.abspath(__file__))
     while f and os.path.basename(f) != "tests": f = os.path.dirname(f)
     f = os.path.join(f, "interop")
     if not os.path.isdir(f):
@@ -36,11 +36,11 @@ test_interop_dir=find_test_interop_dir()
 
 class InteropTest(common.Test):
 
-    def setup(self):
+    def setUp(self):
         self.data = Data()
         self.message = Message()
 
-    def teardown(self):
+    def tearDown(self):
         self.data = None
 
     def get_data(self, name):
@@ -63,6 +63,8 @@ class InteropTest(common.Test):
         # Re-encode and verify pre-computed and actual encoded size match.
         reencoded = self.data.encode()
         assert encoded_size == len(reencoded), "%d != %d" % (encoded_size, len(reencoded))
+        # verify round trip bytes
+        assert reencoded == encoded, "Value mismatch: %s != %s" % (reencoded, encoded)
 
     def decode_message_file(self, name):
         self.message.decode(self.get_data(name))
