@@ -27,8 +27,15 @@ from .common import pump, Skipped
 from proton._compat import str2bin
 
 def _sslCertpath(file):
-    """ Return the full path to the certificate,keyfile, etc.
+    """ Return the full path to the certificate, keyfile, etc.
     """
+    if os.name=="nt":
+        if file.find("private-key")!=-1:
+            # The private key is not in a separate store
+            return None
+        # Substitute pkcs#12 equivalent for the CA/key store
+        if file.endswith(".pem"):
+            file = file[:-4] + ".p12"
     return os.path.join(os.path.dirname(__file__),
                         "ssl_db/%s" % file)
 
@@ -73,9 +80,18 @@ def _testSaslMech(self, mech, clientUser='user@proton', authUser='user@proton',
 class Test(common.Test):
   pass
 
+def consumeAllOuput(t):
+  stops = 0
+  while stops<1:
+    out = t.peek(1024)
+    l = len(out) if out else 0
+    t.pop(l)
+    if l <= 0:
+      stops += 1
+
 class SaslTest(Test):
 
-  def setup(self):
+  def setUp(self):
     self.t1 = Transport()
     self.s1 = SASL(self.t1)
     self.t2 = Transport(Transport.SERVER)
@@ -85,109 +101,97 @@ class SaslTest(Test):
   def pump(self):
     pump(self.t1, self.t2, 1024)
 
-  # Note that due to server protocol autodetect, there can be no "pipelining"
-  # of protocol frames from the server end only from the client end.
-  #
-  # This is because the server cannot know which protocol layers are active
-  # and therefore which headers need to be sent,
-  # until it sees the respective protocol headers from the client.
-  def testPipelinedClient(self):
-    if "java" in sys.platform:
-      raise Skipped("Proton-J does not support client pipelining")
-
-    # Client
-    self.s1.allowed_mechs('ANONYMOUS')
+  # We have to generate the client frames manually because proton does not
+  # generate pipelined SASL and AMQP frames together
+  def testIllegalProtocolLayering(self):
     # Server
     self.s2.allowed_mechs('ANONYMOUS')
 
-    assert self.s1.outcome is None
-    assert self.s2.outcome is None
-
-    # Push client bytes into server
-    out1 = self.t1.peek(1024)
-    self.t1.pop(len(out1))
-    self.t2.push(out1)
-
-    out2 = self.t2.peek(1024)
-    self.t2.pop(len(out2))
-
-    assert self.s1.outcome is None
-
-    self.t1.push(out2)
-
-    assert self.s1.outcome == SASL.OK
-    assert self.s2.outcome == SASL.OK
-
-  def testPipelinedClientFail(self):
-    if "java" in sys.platform:
-      raise Skipped("Proton-J does not support client pipelining")
-
-    # Client
-    self.s1.allowed_mechs('ANONYMOUS')
-    # Server
-    self.s2.allowed_mechs('PLAIN DIGEST-MD5 SCRAM-SHA-1')
-
-    assert self.s1.outcome is None
-    assert self.s2.outcome is None
-
-    # Push client bytes into server
-    out1 = self.t1.peek(1024)
-    self.t1.pop(len(out1))
-    self.t2.push(out1)
-
-    out2 = self.t2.peek(1024)
-    self.t2.pop(len(out2))
-
-    assert self.s1.outcome is None
-
-    self.t1.push(out2)
-
-    assert self.s1.outcome == SASL.AUTH
-    assert self.s2.outcome == SASL.AUTH
-
-  def testSaslAndAmqpInSingleChunk(self):
-    if "java" in sys.platform:
-      raise Skipped("Proton-J does not support client pipelining")
-
-    self.s1.allowed_mechs('ANONYMOUS')
-    self.s2.allowed_mechs('ANONYMOUS')
-
-    # do some work to generate AMQP data
-    c1 = Connection()
     c2 = Connection()
-    self.t1.bind(c1)
-    c1._transport = self.t1
     self.t2.bind(c2)
-    c2._transport = self.t2
 
-    c1.open()
+    assert self.s2.outcome is None
 
-    # get all t1's output in one buffer then pass it all to t2
-    out1_sasl_and_amqp = str2bin("")
-    t1_still_producing = True
-    while t1_still_producing:
-      out1 = self.t1.peek(1024)
-      self.t1.pop(len(out1))
-      out1_sasl_and_amqp += out1
-      t1_still_producing = out1
+    # Push client bytes into server
+    self.t2.push(str2bin(
+        # SASL
+        'AMQP\x03\x01\x00\x00'
+        # @sasl-init(65) [mechanism=:ANONYMOUS, initial-response=b"anonymous@fuschia"]
+        '\x00\x00\x002\x02\x01\x00\x00\x00SA\xd0\x00\x00\x00"\x00\x00\x00\x02\xa3\x09ANONYMOUS\xa0\x11anonymous@fuschia'
+        # SASL (again illegally)
+        'AMQP\x03\x01\x00\x00'
+        # @sasl-init(65) [mechanism=:ANONYMOUS, initial-response=b"anonymous@fuschia"]
+        '\x00\x00\x002\x02\x01\x00\x00\x00SA\xd0\x00\x00\x00"\x00\x00\x00\x02\xa3\x09ANONYMOUS\xa0\x11anonymous@fuschia'
+        # AMQP
+        'AMQP\x00\x01\x00\x00'
+        # @open(16) [container-id="", channel-max=1234]
+        '\x00\x00\x00!\x02\x00\x00\x00\x00S\x10\xd0\x00\x00\x00\x11\x00\x00\x00\x0a\xa1\x00@@`\x04\xd2@@@@@@'
+        ))
 
-    t2_still_consuming = True
-    while t2_still_consuming:
-      num = min(self.t2.capacity(), len(out1_sasl_and_amqp))
-      self.t2.push(out1_sasl_and_amqp[:num])
-      out1_sasl_and_amqp = out1_sasl_and_amqp[num:]
-      t2_still_consuming = num > 0 and len(out1_sasl_and_amqp) > 0
+    consumeAllOuput(self.t2)
 
-    assert len(out1_sasl_and_amqp) == 0, (len(out1_sasl_and_amqp), out1_sasl_and_amqp)
+    assert self.t2.condition
+    assert self.t2.closed
+    assert not c2.state & Endpoint.REMOTE_ACTIVE
 
-    # check that t2 processed both the SASL data and the AMQP data
+  def testPipelinedClient(self):
+    # Server
+    self.s2.allowed_mechs('ANONYMOUS')
+
+    c2 = Connection()
+    self.t2.bind(c2)
+
+    assert self.s2.outcome is None
+
+    # Push client bytes into server
+    self.t2.push(str2bin(
+        # SASL
+        'AMQP\x03\x01\x00\x00'
+        # @sasl-init(65) [mechanism=:ANONYMOUS, initial-response=b"anonymous@fuschia"]
+        '\x00\x00\x002\x02\x01\x00\x00\x00SA\xd0\x00\x00\x00"\x00\x00\x00\x02\xa3\x09ANONYMOUS\xa0\x11anonymous@fuschia'
+        # AMQP
+        'AMQP\x00\x01\x00\x00'
+        # @open(16) [container-id="", channel-max=1234]
+        '\x00\x00\x00!\x02\x00\x00\x00\x00S\x10\xd0\x00\x00\x00\x11\x00\x00\x00\x0a\xa1\x00@@`\x04\xd2@@@@@@'
+        ))
+
+    consumeAllOuput(self.t2)
+
+    assert not self.t2.condition
     assert self.s2.outcome == SASL.OK
     assert c2.state & Endpoint.REMOTE_ACTIVE
 
-  def testPipelined2(self):
-    if "java" in sys.platform:
-      raise Skipped("Proton-J does not support client pipelining")
+  def testPipelinedServer(self):
+    # Client
+    self.s1.allowed_mechs('ANONYMOUS')
 
+    c1 = Connection()
+    self.t1.bind(c1)
+
+    assert self.s1.outcome is None
+
+    # Push server bytes into client
+    # Commented out lines in this test are where the client input processing doesn't
+    # run after output processing even though there is input waiting
+    self.t1.push(str2bin(
+        # SASL
+        'AMQP\x03\x01\x00\x00'
+        # @sasl-mechanisms(64) [sasl-server-mechanisms=@PN_SYMBOL[:ANONYMOUS]]
+        '\x00\x00\x00\x1c\x02\x01\x00\x00\x00S@\xc0\x0f\x01\xe0\x0c\x01\xa3\tANONYMOUS'
+        # @sasl-outcome(68) [code=0]
+        '\x00\x00\x00\x10\x02\x01\x00\x00\x00SD\xc0\x03\x01P\x00'
+        # AMQP
+        'AMQP\x00\x01\x00\x00'
+        # @open(16) [container-id="", channel-max=1234]
+        '\x00\x00\x00!\x02\x00\x00\x00\x00S\x10\xd0\x00\x00\x00\x11\x00\x00\x00\x0a\xa1\x00@@`\x04\xd2@@@@@@'
+        ))
+
+    consumeAllOuput(self.t1)
+
+    assert self.s1.outcome == SASL.OK
+    assert c1.state & Endpoint.REMOTE_ACTIVE
+
+  def testPipelined2(self):
     out1 = self.t1.peek(1024)
     self.t1.pop(len(out1))
     self.t2.push(out1)
@@ -220,7 +224,7 @@ class SaslTest(Test):
     out = self.t1.peek(1024)
     self.t1.pop(len(out))
 
-    self.t1.push(str2bin("6\x02\x01\x00\x00\x00S@\xc04\x01\xe01\x04\xa3\x05PLAIN\x0aDIGEST-MD5\x09ANONYMOUS\x08CRAM-MD5"))
+    self.t1.push(str2bin("6\x02\x01\x00\x00\x00S@\xc0\x29\x01\xe0\x26\x04\xa3\x05PLAIN\x0aDIGEST-MD5\x09ANONYMOUS\x08CRAM-MD5"))
     out = self.t1.peek(1024)
     self.t1.pop(len(out))
     self.t1.push(str2bin("\x00\x00\x00\x10\x02\x01\x00\x00\x00SD\xc0\x03\x01P\x00"))
@@ -276,8 +280,6 @@ class SaslTest(Test):
     assert self.t1.condition != None
 
   def testMechNotFound(self):
-    if "java" in sys.platform:
-      raise Skipped("Proton-J does not support checking authentication state")
     self.c1 = Connection()
     self.c1.open()
     self.t1.bind(self.c1)
@@ -291,7 +293,7 @@ class SaslTest(Test):
     assert self.s2.outcome != SASL.OK
 
 class SASLMechTest(Test):
-  def setup(self):
+  def setUp(self):
     self.t1 = Transport()
     self.s1 = SASL(self.t1)
     self.t2 = Transport(Transport.SERVER)
@@ -351,7 +353,7 @@ class SASLMechTest(Test):
     _testSaslMech(self, 'PLAIN')
 
 # SCRAM not supported before Cyrus SASL 2.1.26
-# so not universal and hance need a test for support
+# so not universal and hence need a test for support
 # to keep it in tests.
 #  def testSCRAMSHA1(self):
 #    common.ensureCanTestExtendedSASL()
@@ -366,7 +368,7 @@ def _sslConnection(domain, transport, connection):
   return connection
 
 class SSLSASLTest(Test):
-  def setup(self):
+  def setUp(self):
     if not common.isSSLPresent():
       raise Skipped("No SSL libraries found.")
 
@@ -382,8 +384,6 @@ class SSLSASLTest(Test):
     self.c2 = Connection()
 
   def testSSLPlainSimple(self):
-    if "java" in sys.platform:
-      raise Skipped("Proton-J does not support SSL with SASL")
     if not SASL.extended():
       raise Skipped("Simple SASL server does not support PLAIN")
     common.ensureCanTestExtendedSASL()
@@ -401,8 +401,6 @@ class SSLSASLTest(Test):
     _testSaslMech(self, mech, encrypted=True)
 
   def testSSLPlainSimpleFail(self):
-    if "java" in sys.platform:
-      raise Skipped("Proton-J does not support SSL with SASL")
     if not SASL.extended():
       raise Skipped("Simple SASL server does not support PLAIN")
     common.ensureCanTestExtendedSASL()
@@ -420,10 +418,10 @@ class SSLSASLTest(Test):
     _testSaslMech(self, mech, clientUser='usr@proton', encrypted=True, authenticated=False)
 
   def testSSLExternalSimple(self):
-    if "java" in sys.platform:
-      raise Skipped("Proton-J does not support SSL with SASL")
-
-    extUser = 'O=Client,CN=127.0.0.1'
+    if os.name=="nt":
+      extUser = 'O=Client, CN=127.0.0.1'
+    else:
+      extUser = 'O=Client,CN=127.0.0.1'
     mech = 'EXTERNAL'
 
     self.server_domain.set_credentials(_sslCertpath("server-certificate.pem"),
@@ -444,9 +442,6 @@ class SSLSASLTest(Test):
     _testSaslMech(self, mech, clientUser=None, authUser=extUser, encrypted=True)
 
   def testSSLExternalSimpleFail(self):
-    if "java" in sys.platform:
-      raise Skipped("Proton-J does not support SSL with SASL")
-
     mech = 'EXTERNAL'
 
     self.server_domain.set_credentials(_sslCertpath("server-certificate.pem"),
@@ -464,8 +459,8 @@ class SSLSASLTest(Test):
     _testSaslMech(self, mech, clientUser=None, authUser=None, encrypted=None, authenticated=False)
 
 class SASLEventTest(engine.CollectorTest):
-  def setup(self):
-    engine.CollectorTest.setup(self)
+  def setUp(self):
+    engine.CollectorTest.setUp(self)
     self.t1 = Transport()
     self.s1 = SASL(self.t1)
     self.t2 = Transport(Transport.SERVER)
